#ifndef _VIAAC97_H_
#define _VIAAC97_H_

#include <dos.h>

static unsigned short viaac97_status_read(unsigned int base, unsigned short *data);
#pragma aux viaac97_status_read = \
	"add dx, 0x80" \
	"in eax, dx" \
	"mov [bx], ax" \
	"shr eax, 16" \
	__parm [dx] [bx] \
	__value [ax] \
	__modify [dx]
	
static unsigned short viaac97_command_write(unsigned int base, unsigned short cmd, unsigned short data);
#pragma aux viaac97_command_write = \
	"add dx, 0x80" \
	"shl ebx, 16" \
	"and eax, 0xFFFF" \
	"or eax, ebx" \
	"out dx, eax" \
	__parm [dx] [bx] [ax] \
	__modify [dx bx]

enum CodecID {
	CODEC_PRIMARY,
	CODEC_SECONDARY
};

static bool viaac97_codec_busy(unsigned int base)
{
	unsigned short data;
	unsigned short status = viaac97_status_read(base, &data);

	return !!(status & 0x100);
}

static bool viaac97_codec_read_valid_data(unsigned int base, enum CodecID codec, unsigned short *data)
{
	unsigned short flag = codec == CODEC_SECONDARY ? 0x800 : 0x200;
	unsigned short status = viaac97_status_read(base, data);

	if (status & 0x100) {
		return false;
	}

	return !!(status & flag);
}

static unsigned short viaac97_codec_read(unsigned int base, enum CodecID codec, unsigned short reg)
{
	unsigned short cmd = 0;
	unsigned short data = 0;
	unsigned int tries = 0;
	
	if (codec == CODEC_SECONDARY)    cmd |= 0x4000 | 0x800;
	else if (codec == CODEC_PRIMARY) cmd |= 0x200;
	
	cmd |= 0x80; // Codec register read mode
	cmd |= (reg & 0x7F); // Register index
	
	for (tries = 0; tries < 3; tries++) {
		data = 0;
		viaac97_command_write(base, cmd, data);

		if (viaac97_codec_read_valid_data(base, codec, &data)) {
			return data;
		}

		delay(1);
		
		if (viaac97_codec_read_valid_data(base, codec, &data)) {
			return data;
		}
		
		delay(2);
	}
	
	printf("Timeout on AC97 read\n");
	return 0xFFFF;
}

enum {
	VIAAC97_MIXER_REG_RESET   = 0,
	VIAAC97_MIXER_REG_MASTER  = 0x02,
	VIAAC97_MIXER_REG_AUX_OUT = 0x04,
	VIAAC97_MIXER_REG_PCBEEP  = 0x0A,
	VIAAC97_MIXER_REG_MIC     = 0x0E,
	VIAAC97_MIXER_REG_LINE_IN = 0x10,
	VIAAC97_MIXER_REG_CD      = 0x12,
	VIAAC97_MIXER_REG_PCM_OUT = 0x18,
};

static void viaac97_codec_write(unsigned int base, enum CodecID codec, unsigned short reg, unsigned short data)
{
	unsigned short cmd = 0;
	unsigned int tries = 0;
	
	if (codec == CODEC_SECONDARY)    cmd |= 0x4000;
	
	cmd |= (reg & 0x7F); // Register index
	
	viaac97_command_write(base, cmd, data);
	
	for (tries = 0; tries < 3; tries++) {
		if (!viaac97_codec_busy(base)) {
			return;
		}

		delay(1);
	}
	
	printf("Timeout on AC97 write\n");
}

#endif
